function fxDTSBrick::doObstructionCheck(%brick, %pos, %masks)
{
	if(%masks $= "")
	{
		%masks = $TypeMasks::FxBrickObjectType;
	}

	%x = firstWord(%pos);
	%y = getWord(%pos, 1);
	%z = getWord(%pos, 2);

	%center = %brick.getPosition();

	%minBox = getWords(%brick.getWorldBox(), 0, 2);
	%minX = firstWord(%minBox);
	%minY = getWord(%minBox, 1);
	%minZ = getWord(%minBox, 2);

	%maxBox = getWords(%brick.getWorldBox(), 3, 5);
	%maxX = firstWord(%maxBox);
	%maxY = getWord(%maxBox, 1);
	%maxZ = getWord(%maxBox, 2);

	%box = vectorSub(%maxBox, %minBox);

	if(containerRayCast(%pos, %minX SPC %y SPC %minZ, %masks, %brick) !$= "0" && containerRayCast(%pos, %minX SPC %y SPC %z, %masks, %brick) !$= "0" && containerRayCast(%pos, %minX SPC %y SPC %maxZ, %masks, %brick) !$= "0")
	{
		if(containerRayCast(%pos, %x SPC %y SPC %minZ, %masks, %brick) !$= "0" && containerRayCast(%pos, %x SPC %y SPC %z, %masks, %brick) !$= "0" && containerRayCast(%pos, %x SPC %y SPC %maxZ, %masks, %brick) !$= "0")
		{
			if(containerRayCast(%pos, %maxX SPC %y SPC %minZ, %masks, %brick) !$= "0" && containerRayCast(%pos, %maxX SPC %y SPC %z, %masks, %brick) !$= "0" && containerRayCast(%pos, %maxX SPC %y SPC %maxZ, %masks, %brick) !$= "0")
			{
				return true;
			}
		}
	}

	return false;
}

function ShapeBaseImageData::isMelee(%data)
{
	if(!isObject(%proj = %data.projectile))
	{
		return false;
	}

	if(%proj.armingDelay <= 0 && %proj.lifetime <= 175 && %proj.bounceElasticity <= 0 && %proj.bounceFriction <= 0 && %proj.gravityMod <= 0.25 && (%proj.directDamage || (isObject(%explo = %proj.explosion) && %explo.radiusDamage && %explo.damageRadius <= 10)))
	{
		return true;
	}

	return false;
}

function eulerToAxis(%euler)
{
	%euler = vectorScale(%euler, $pi / 180);
	%matrix = matrixCreateFromEuler(%euler);
	return getWords(%matrix, 3, 6);
}

function axisToEuler(%axis)
{
	%angleOver2 = getWord(%axis, 3) * 0.5;
	%angleOver2 = -%angleOver2;

	%sinThetaOver2 = mSin(%angleOver2);
	%cosThetaOver2 = mCos(%angleOver2);

	%q0 = %cosThetaOver2;
	%q1 = firstWord(%axis) * %sinThetaOver2;
	%q2 = getWord(%axis, 1) * %sinThetaOver2;
	%q3 = getWord(%axis, 2) * %sinThetaOver2;

	%q0q0 = %q0 * %q0;
	%q1q2 = %q1 * %q2;
	%q0q3 = %q0 * %q3;
	%q1q3 = %q1 * %q3;
	%q0q2 = %q0 * %q2;
	%q2q2 = %q2 * %q2;
	%q2q3 = %q2 * %q3;
	%q0q1 = %q0 * %q1;
	%q3q3 = %q3 * %q3;

	%m13 = 2.0 * (%q1q3 - %q0q2);
	%m21 = 2.0 * (%q1q2 - %q0q3);
	%m22 = 2.0 * %q0q0 - 1.0 + 2.0 * %q2q2;
	%m23 = 2.0 * (%q2q3 + %q0q1);
	%m33 = 2.0 * %q0q0 - 1.0 + 2.0 * %q3q3;

	return mRadToDeg(mAsin(%m23)) SPC mRadToDeg(mAtan(-%m13, %m33)) SPC mRadToDeg(mAtan(-%m21, %m22));
}

function SimObject::call(%this,%method,%v0,%v1,%v2,%v3,%v4,%v5,%v6,%v7,%v8,%v9,%v10,%v11,%v12,%v13,%v14,%v15,%v16,%v17)
{
	%lastNull = -1;
	for(%i = 0; %i < 18; %i ++)
	{
		%a = %v[%i];
		if(%a $= "")
		{
			if(%lastNull < 0)
				%lastNull = %i;
			continue;
		}
		else
		{
			if(%lastNull >= 0)
			{
				for(%e = %lastNull; %e < %i; %e ++)
				{
					if(%args !$= "")
						%args = %args @ ",";
					%args = %args @ "\"\"";
				}
				%lastNull = -1;
			}
			if(%args !$= "")
				%args = %args @ ",";
			%args = %args @ "\"" @ %a @ "\"";
		}
	}

	eval(%this @ "." @ %method @ "(" @ %args @ ");");
}

$ChainBatchSize = 100;
$ChainTimeOut = 10;

function SimGroup::chainMethodCall(%this,%method,%v0,%v1,%v2,%v3,%v4,%v5,%v6,%v7,%v8,%v9,%v10,%v11,%v12,%v13,%v14,%v15,%v16,%v17)
{
	%batch = (%this.chain_batchSize $= "" ? $Slayer::Server::ChainBatchSize : %this.chain_batchSize);
	%count = %this.getCount();
	%index = (%this.chain_index $= "" ? %count - 1 : %this.chain_index);
	%endIndex = (%index - %batch < 0 ? 0 : %index - %batch);

	for(%i = %index; %i >= %endIndex; %i --)
	{
		%obj = %this.getObject(%i);
		%obj.call(%method,%v0,%v1,%v2,%v3,%v4,%v5,%v6,%v7,%v8,%v9,%v10,%v11,%v12,%v13,%v14,%v15,%v16,%v17);
	}
	%this.chain_index = %endIndex - 1;
	if(%this.chain_index <= 0)
	{
		if(isFunction(%this,%this.chain_callback))
			%this.call(%this.chain_callback);
		%this.chain_index = "";
		%this.chain_batchSize = "";
		%this.chain_timeOut = "";
		%this.chain_callback = "";
	}
	else
	{
		cancel(%this.chain_schedule);
		%time = (%this.chain_timeOut $= "" ? $Slayer::Server::ChainTimeOut : %this.chain_timeOut);
		%this.chain_schedule = %this.schedule(%time,"chainMethodCall",%method,%v0,%v1,%v2,%v3,%v4,%v5,%v6,%v7,%v8,%v9,%v10,%v11,%v12,%v13,%v14,%v15,%v16,%v17);
	}
}

function BaseRaiders_findClosestColor(%x)
{
	%x = firstWord(%x) / 255 SPC getWord(%x, 1) / 255 SPC getWord(%x, 2) / 255 SPC getWord(%x, 3) / 255;

	for(%a = 0; %a < 64; %a++)
	{
		%match = mAbs(firstWord(getColorIDTable(%a)) - firstWord(%x)) + mAbs(getWord(getColorIDTable(%a), 1) - getWord(%x, 1)) + mAbs(getWord(getColorIDTable(%a), 2) - getWord(%x, 2)) + mAbs(getWord(getColorIDTable(%a), 3) - getWord(%x, 3)) * 4;

		if(%match < %bestMatch || %bestMatch $= "")
		{
			%bestMatch = %match;
			%bestID = %a;
		}
	}

	return %bestID;
}

function mRound(%num)
{
	return mFloor(%num + 0.5);
}

function minClamp(%num, %min)
{
	if(%num <= %min)
	{
		return %min;
	}

	return %num;
}

function maxClamp(%num, %max)
{
	if(%num >= %max)
	{
		return %max;
	}

	return %num;
}

function isNumber(%string) //Function made by Clockturn.
{
	if(%string <= 999999 && (%string >= -999999))
	{
		// Fast solution
		return %string $= %string * 1;
	}

	else
	{
		// Slow solution
		%dot = 0; // Track if period present
		%sci = 0; // Track if scientific notation present

		for(%j = 0; %j < strLen(%string); %j++)
		{
			%c = getSubStr(%string, %j, 1);

			if(%c $= "-")
			{
				if(%sci == -1)
				{
					%sci = 1; // Negative powers
				}

				else if(%j != 0)
				{
					return false; // Negative not the first character
				}

				continue;
			}

			if(%c $= ".")
			{
				if(%dot || %j == 0)
				{
					return false; // Multiple periods or nothing before the period
				}

				else
				{
					%dot = 1;
				}

				continue;
			}

			if(%c $= "e")
			{
				if(%sci != 0)
				{
					return false; // Multiple Es present
				}

				else
				{
					%sci = -1;
				}

				continue;
			}

			if(%c $= "+")
			{
				if(%sci != -1)
				{
					return false; // + is not part of e+ scientific notation
				}

				else
				{
					%sci = 1;
				}

				continue;
			}

			if(%sci == -1 || strPos("0123456789", %c) == -1)
			{
				return false; // An E was present without a + or the current character is not a number
			}
		}

		if(%sci == -1)
		{
			return false; // Last character was an E.
		}

		return true; // Gamut has been run, nothing returned, ergo it is fine
	}
}

//List functions found in Event_Minigame
function addItemToList(%string, %item)
{
	if(hasItemOnList(%string, %item))
	{
		return %string;
	}

	if(%string $= "")
	{
		return %item;
	}

	else
	{
		return %string SPC %item;
	}
}

function hasItemOnList(%string, %item)
{
	for(%i = 0; %i < getWordCount(%string); %i++)
	{
		if(getWord(%string, %i) $= %item)
		{
			return 1;
		}
	}

	return 0;
}

function removeItemFromList(%string, %item)
{
	if(!hasItemOnList(%string, %item))
	{
		return %string;
	}

	for(%i = 0; %i < getWordCount(%string); %i++)
	{
		if(getWord(%string, %i) $= %item)
		{
			if(%i == 0)
			{
				return getWords(%string,1,getWordCount(%string));
			}

			else if(%i == getWordCount(%string) - 1)
			{
				return getWords(%string, 0, %i - 1);
			}

			else
			{
				return getWords(%string, 0, %i - 1) SPC getWords(%string, %i + 1, getWordCount(%string));
			}
		}
	}
}

function fxDTSBrick::setPrintByName(%brick, %print)
{
	switch$(strReplace(strLwr(%print), " ", ""))
	{
		case "" or "space":
			%print = "-space";

		case "?" or "qmark" or "questionmark":
			%print = "-qmark";

		case "&" or "and":
			%print = "-and";

		case "!" or "bang" or "exclamation" or "exclamationmark":
			%print = "-bang";

		case "@" or "at":
			%print = "-at";

		case "*" or "asterisk" or "star" or "multiply":
			%print = "-asterisk";

		case "'" or "apostrophe" or "apostro" or "apostroph" or "apost":
			%print = "-apostrophe";

		case "^" or "caret" or "up" or "uparrow":
			%print = "-caret";

		case "$" or "dollar" or "dollarsymbol" or "money" or "cash":
			%print = "-dollar";

		case "=" or "equals":
			%print = "-equals";

		case ">" or "greaterthan" or "largerthan" or "biggerthan" or "rightarrow":
			%print = "-greater_than";

		case "<" or "lessthan" or "smallerthan" or "leftarrow":
			%print = "-less_than";

		case "-" or "minus" or "subtract" or "dash":
			%print = "-minus";

		case "%" or "percent" or "ratio" or "modulo":
			%print = "-percent";

		case "." or "period" or "dot":
			%print = "-period";

		case "+" or "plus" or "add":
			%print = "-plus";

		case "#" or "pound" or "hash" or "hashtag" or "numbersymbol" or "numbersign" or "number":
			%print = "-pound";

		case "zero" or "0":
			%brick.setPrintCount(0);

			return;

		case "one" or "1":
			%brick.setPrintCount(1);

			return;

		case "two" or "2":
			%brick.setPrintCount(2);

			return;

		case "three" or "3":
			%brick.setPrintCount(3);

			return;

		case "four" or "4":
			%brick.setPrintCount(4);

			return;

		case "five" or "5":
			%brick.setPrintCount(5);

			return;

		case "six" or "6":
			%brick.setPrintCount(6);

			return;

		case "seven" or "7":
			%brick.setPrintCount(7);

			return;

		case "eight" or "8":
			%brick.setPrintCount(8);

			return;

		case "nine" or "9":
			%brick.setPrintCount(9);

			return;
	}

	%count = getNumPrintTextures();

	for(%i = 0; %i < %count; %i++)
	{
		%this = fileBase(getPrintTexture(%i));

		if(%this $= %print)
		{
			%brick.setPrint(%i);
		}
	}
}

function getPrintIDByName(%print)
{
	switch$(strReplace(strLwr(%print), " ", ""))
	{
		case "" or "space":
			%print = "-space";

		case "?" or "qmark" or "questionmark":
			%print = "-qmark";

		case "&" or "and":
			%print = "-and";

		case "!" or "bang" or "exclamation" or "exclamationmark":
			%print = "-bang";

		case "@" or "at":
			%print = "-at";

		case "*" or "asterisk" or "star" or "multiply":
			%print = "-asterisk";

		case "'" or "apostrophe" or "apostro" or "apostroph" or "apost":
			%print = "-apostrophe";

		case "^" or "caret" or "up" or "uparrow":
			%print = "-caret";

		case "$" or "dollar" or "dollarsymbol" or "money" or "cash":
			%print = "-dollar";

		case "=" or "equals":
			%print = "-equals";

		case ">" or "greaterthan" or "largerthan" or "biggerthan" or "rightarrow":
			%print = "-greater_than";

		case "<" or "lessthan" or "smallerthan" or "leftarrow":
			%print = "-less_than";

		case "-" or "minus" or "subtract" or "dash":
			%print = "-minus";

		case "%" or "percent" or "ratio" or "modulo":
			%print = "-percent";

		case "." or "period" or "dot":
			%print = "-period";

		case "+" or "plus" or "add":
			%print = "-plus";

		case "#" or "pound" or "hash" or "hashtag" or "numbersymbol" or "numbersign" or "number":
			%print = "-pound";

		case "zero" or "0":
			%print = "0";

			return;

		case "one" or "1":
			%print = "1";

			return;

		case "two" or "2":
			%print = "2";

			return;

		case "three" or "3":
			%print = "3";

			return;

		case "four" or "4":
			%print = "4";

			return;

		case "five" or "5":
			%print = "5";

			return;

		case "six" or "6":
			%print = "6";

			return;

		case "seven" or "7":
			%print = "7";

			return;

		case "eight" or "8":
			%print = "8";

			return;

		case "nine" or "9":
			%print = "9";

			return;
	}

	%count = getNumPrintTextures();

	for(%i = 0; %i < %count; %i++)
	{
		%this = fileBase(getPrintTexture(%i));

		if(%this $= %print)
		{
			return %i;
		}

		else if(%this $= "-space")
		{
			%spaceID = %i;
		}
	}

	return %spaceID;
}

function fxDTSBrick::createBaseRaidersTrigger(%brick, %data)
{
	//credits to Space Guy for showing how to create triggers

	if(!isObject(%data) || %data.getClassName() !$= "triggerData")
	{
		return;
	}

	%t = new trigger()
	{
		datablock = %data;
		polyhedron = "0 0 0 1 0 0 0 -1 0 0 0 1"; //this determines the shape of the trigger
		brick = %brick;
	};

	missionCleanup.add(%t);
	
	%boxMax = getWords(%brick.getWorldBox(), 3, 5);
	%boxMin = getWords(%brick.getWorldBox(), 0, 2);
	%boxDiff = vectorSub(%boxMax, %boxMin);
	%t.setScale(%boxDiff);

	%posA = %brick.getWorldBoxCenter();
	%posB = %t.getWorldBoxCenter();
	%posDiff = vectorSub(%posA, %posB);
	%t.setTransform(%posDiff);

	//All this makes a trigger that's the same size as the brick on all
	//sides. Most functions that create triggers make it a tiny bit
	//bigger than the brick on all sides.

	%brick.trigger = %t;

	return %t;
}